package WiKIDUpdate;

use vars qw( %Config $modulus $release $build $hostname $ipaddr $quality );

sub read_file ($) {
   my ($file) = @_;
   my @ret;
   unless (open (READFILE, $file)) {
      return ();
   }
   while (my $line = <READFILE>) {
      chomp($line);
      if ($line) {
         push @ret, $line;
      }
   }
   close(READFILE);
   return (@ret);
}

foreach (&read_file('/opt/WiKID/conf/update.conf')) {
   next if /^\s*#/;
   my ($name, $val);
   if (($name, $val) = /^([^=]+)=(.+)$/) {
      $Config{$name} = $val;
   }
}

($modulus) = &read_file('/opt/WiKID/var/update/cert_modulus_md5sum');
($release) = &read_file('/etc/WiKID/version');
($build) = &read_file('/opt/WiKID/var/update/curr_build');
$hostname = `/bin/hostname -f`;
chomp($hostname);
$ipaddr = `ifconfig eth0 2>/dev/null | awk '/inet addr:/ {gsub("addr:", "", \$2); print \$2}' 2>/dev/null`;
chomp ($ipaddr);
unless ($ipaddr) {
   $ipaddr = `ifconfig eth1 2>/dev/null | awk '/inet addr:/ {gsub("addr:", "", \$2); print \$2}' 2>/dev/null`;
   chomp ($ipaddr);
}
$quality = "Production";
if ($Config{'QUALITY'}) {
   $quality = $Config{'QUALITY'};
}

$release = 'UNKNOWN' unless $release;
$build = 'UNKNOWN' unless $build;
$hostname = 'UNKNOWN' unless $hostname;
$ipaddr = 'UNKNOWN' unless $ipaddr;
$modulus = 'UNKNOWN' unless $modulus;

sub get_lines_from_server ($) {
   my ($URL) = @_;
   my @output;
   unless (open (SERVER, "/usr/bin/curl -s -S -m 120 --cacert /opt/WiKID/private/WiKIDCA.cer '$URL' </dev/null 2>&1 |")) {
      die "Could not open connection to server: $!\n";
   }

   while (my $line = <SERVER>) {
      if ($line =~ s/^curl: \(\d+\)//) {
         die "Error communicating with update server: $line";
      } elsif ($line =~ s/^SERVER ERROR: //) {
         die "Error occured on update server: $line";
      } else {
         push @output, $line;
      }
   }
   close(SERVER);
   return (@output);
}

my $ua;
sub prepare_client_auth ($) {
   my ($passphrase) = @_;
   # Set these for use by the HTTPS connection
   $ENV{'HTTPS_PKCS12_FILE'} = '/opt/WiKID/var/update/update_cert.p12';
   $ENV{'HTTPS_PKCS12_PASSWORD'} = $passphrase;
   $ENV{'HTTPS_CA_FILE'} = '/opt/WiKID/private/WiKIDCA.cer';

   use LWP::UserAgent;
   $ua = new LWP::UserAgent;
   $ua->agent('WiKIDUpdate/1.0');
}

sub get_ssl_file ($$) {
   my ($URL, $outfile, $size) = @_;
   if (-f $outfile and (-s $outfile == $size)) {
      # File already present
      return 1;
   }
   print "Downloading $outfile ($size bytes)\n";
   my $req = new HTTP::Request('GET', $URL);
   my $res = $ua->request($req, $outfile);
   if ($res->is_error) {
      print STDERR $res->error_as_HTML;
      return 0;
   }
   if (-f $outfile and (-s $outfile == $size)) {
      # File downloaded properly
      return 1;
   }
   print STDERR "Downloaded file not of correct size: $URL\n";
   return 0;
}

sub verify_update_dir ($) {
   my ($dir) = @_;
   open(ORIG, "$dir/orig_release") or return 0;
   my $orig_release = <ORIG>;
   chomp($orig_release);
   close(ORIG);
   unless ($orig_release eq $release) {
      return 0;
   }
   open(ORIG, "$dir/orig_build") or return 0;
   my $orig_build = <ORIG>;
   chomp($orig_build);
   close(ORIG);
   unless ($orig_build eq $build) {
      return 0;
   }
   return 1;
}

sub get_rpm_info ($) {
   my ($file) = @_;
   my $response = `rpm -q --qf '%{name} - %{version} - %{release}' -p '$file'`;
   chomp($response);
   return () unless $response;
   return (split(/ - /, $response));
}

sub load_rpms_from_directory ($) {
   my ($dir) = @_;
   my @ret;
   unless (opendir(DIR, $dir)) {
      return ();
   }
   while (my $entry = readdir(DIR)) {
      next if ($entry =~ /^\./);
      if (-d "$dir/$entry") {
         push @ret, &load_rpms_from_directory("$dir/$entry");
      } elsif ($entry =~ /\.rpm$/) {
         my @data = &get_rpm_info("$dir/$entry");
         if (@data) {
            my $info;
            $$info{'name'} = $data[0];
            $$info{'version'} = $data[1];
            $$info{'release'} = $data[2];
            $$info{'file'} = "$dir/$entry";
            push @ret, $info;
         }
      }
   }
   closedir(DIR);
   return (@ret);
}

sub install_directory {
   my ($dir, @options) = @_;
   my @files = ();
   foreach (&load_rpms_from_directory($dir)) {
      my ($version, $release) = &get_curr_version($$_{'name'});
      if (($version ne $$_{'version'}) or ($release ne $$_{'release'})) {
         push @files, $$_{'file'};
      }
   }
   if (@files) {
      unless (system('rpm', '-Uvh', '--oldpackage', '--nodeps', @options, @files) == 0) {
         return 0;
      }
      foreach (@files) {
         &logit("Package $_ has been installed");
      }
      return 1;
   } else {
      return -1;
   }
}

sub get_curr_version ($) {
   my ($pkgname) = @_;
   my $response = `rpm -q --qf '%{version}-%{release}' '$pkgname'`;
   chomp($response);
   return () unless $response;
   return (split(/-/, $response));
}

sub remove_packages {
   my @remove_list = ();
   foreach (@_) {
      if (&get_curr_version($_)) {
         push @remove_list, $_;
      }
   }
   if (@remove_list) {
      unless (system('rpm', '-e', '--nodeps', @remove_list) == 0) {
         return 0;
      }
      foreach (@remove_list) {
         &logit("Package $_ has been removed");
      }
      return 1;
   } else {
      return -1;
   }
}

sub logit ($) {
   my ($entry) = @_;
   my $now = localtime;
   if (open (LOG, ">>/opt/WiKID/var/upgrade.log")) {
      print LOG "$now $entry\n";
      close(LOG);
   }
}

sub record_release_build ($$) {
   my ($release, $build) = @_;
   unless (open (VERSION, ">/etc/WiKID/version")) {
      die "Could not modify /etc/WiKID/version!\n";
   }
   print VERSION "$release\n";
   close(VERSION);

   unless (open (BUILD, ">/opt/WiKID/var/update/curr_build")) {
      die "Could not modify /opt/WiKID/var/update/curr_build!\n";
   }
   print BUILD "$build\n";
   close(BUILD);

   unless (open (RHRELEASE, ">/etc/fedora-release")) {
      die "Could not modify /etc/fedora-release!\n";
   }
   print RHRELEASE "WiKID Linux release $release, build $build\n";
   close(RHRELEASE);

   &logit("Release $release, build $build has been installed");
}

1;
