#!/usr/bin/perl -w

use strict;
use File::Copy;

my $pgroot = '/etc/postgresql';
my $pgversion = '0';
my $DEBUG = 0;

my @WIKID_HOME= split('=', `grep WIKID_HOME= /etc/WiKID/setenv.sh`);
my $WIKID_HOME = $WIKID_HOME[1];
chomp $WIKID_HOME;

my @REQUIRED_HBA = (
	[ qw(local  wikid    all  trust) ],
	[ qw(local  template1    all  trust) ],
	[ qw(local  postgres    postgres  trust) ],
	[ qw(local  all    all  trust) ],
	[ qw(host   wikid    all  127.0.0.1/24   trust) ],
	[ qw(host   wikid 	 all  ::1/128        trust) ],
	[ qw(host   template1    all  127.0.0.1/24   trust) ],
	[ qw(host   template1 	 all  ::1/128        trust) ],
	[ qw(host   all 	 all  ::1/128        trust) ],
	[ qw(host   all    all  127.0.0.1/24   trust) ],
);

sub edit_pgconf;
sub edit_hba;
sub restart_postgres;  # needed to make pg_hb changes take effect
sub check_version;
sub dprint($);

print "Updating PostgreSQL configuration files for use with WiKID ...\n";

if ( -f "/etc/redhat-release" )
{
	dprint "Updating RedHat-based version of PostgreSQL ...";
	$pgroot = '/var/lib/pgsql/data';
}
elsif ( -f "/etc/debian_version" )
{
	dprint "Updating debian version of PostgreSQL ...";
	$pgroot = '';
	if (-x '/usr/bin/pg_lsclusters') {
		my @dbs = `/usr/bin/pg_lsclusters | grep 5432`;
		if (scalar(@dbs)>0) {
			my @fields = split(/\s+/, $dbs[0]);
			$pgversion = $fields[0];
			$pgroot = "/etc/postgresql/$pgversion/main";
		}
	}
	if (length($pgroot) == 0) {
		print "Unable to find an active PostgreSQL listener on port 5432!\n";
		exit 1;

	}
}
else
{
	print "Where is your PostgreSQL config directory?\n";
	exit 1;
}

unless ( -d $pgroot ) { die "Directory '$pgroot' does not exist!\n"; }

&edit_pgconf;
&edit_hba;
&restart_postgres;

## subroutines ################################################################

sub edit_pgconf
{
	# Check which version of Pg we're using, so we can modify the conf 
	# appropriately.  For 7.x, we need tcpip_sockets on.  For 8.x, 
	# sockets are on by default, but the parameter is deprecated while
	# its presence will prevent it from starting!  

	my $conf_file = "$pgroot/postgresql.conf";

	my $ver = check_version;
	dprint "I think I'm editing version $ver ...";

	open(rFILE, "$conf_file") or die "Unable to open '$conf_file': $!\n";
	my @lines = <rFILE>;
	close(rFILE);

	my @newlines = ();
	if ($ver =~ /^7\./)
	{
		dprint "Editing 7.x config file ...";

		foreach my $line (@lines)
		{
			if (grep /^\#?\s*tcpip_socket\s*=/, $line)
			{
				$line = "tcpip_socket = true";	
			}
			elsif (grep /^\#?\s*syslog\s*=/, $line)
			{
				$line = "syslog = 1";	
			}
				
			push @newlines, $line;
		}
				
		return;
	}
	elsif ($ver =~ /^(8|9)\./)
	{
		dprint "Editing v8 or v9 config file ...";

		foreach my $line (@lines)
		{
			# remove 7.x parameters, if found
			next if (grep /^\#?\s*tcpip_socket\s*=/, $line);
			next if (grep /^\#?\s*syslog\s*=/, $line);

			if (grep /^\#\s*listen_addresses\s*=/, $line)
			{
				$line = "listen_addresses = localhost";	
			}
			if (grep /^\#\s*log_destination\s*=/, $line)
			{
				$line = "log_destination = 'syslog'";	
			}
			push @newlines, $line;
		}
	}
		
	if (scalar @newlines > 0)
	{
		rename($conf_file, "${conf_file}.pre-wikid");
		open(wFILE, ">", "${conf_file}") or die "Unable to open '$conf_file' for writing: $!\n";
		print wFILE @newlines;
		close(wFILE);
		return;
	}

	print "Unable to determine which version of Pg you have. No changes made.";
}

sub check_version
{
	return $pgversion if $pgversion ne '0';
	my $version = '';
	my @cmds = qw(postmaster pg_config psql);
	foreach my $cmd (@cmds)
	{
		my @which = `which $cmd`;
		next unless @which;
		dprint "Location: @which";	
		my @output = `$cmd --version`;
		dprint "OUTPUT: @output";	
		$version = $output[0];
		$version =~ s/.*(\d+\.\d+\.\d+).*/$1/;
		last if @output;
	}
	
	chomp $version;
	dprint "Version: $version\n";

	return $version;
}

sub edit_hba
{
	my $hba_file = "$pgroot/pg_hba.conf";

	dprint "Checking for the presence of the following values:";
	foreach my $a (@REQUIRED_HBA)
	{
		dprint "@{$a}";
	}

	my @newlines = ();
	my %seen = ();
	my @toadd = ();
	my $last_blank_line = 0;
	my $insert_ndx = 0;
	my @lines = ();
	if ( -f "$hba_file" )
	{
		open(rFILE, "$hba_file") or die "Unable to open '$hba_file': $!\n";
		@lines = <rFILE>;
		close(rFILE);

		my $lineno = 0;
		foreach my $line (@lines)
		{
			chomp $line;
			$lineno++;
			$last_blank_line = $lineno if $line =~ /^\s*$/ && $insert_ndx == 0; # save marker so we can prepend there
			next if $line =~ /^\s*#/ || $line =~ /^\s*$/;
			$insert_ndx = $lineno if $insert_ndx == 0; # save marker so we can prepend there
			#print $line;
			my @fields = split /\s+/,  $line;
			#dprint "@fields";

			my $key = join "-", @fields;
			$seen{$key} = 1;

			# need to check for variations on the address, which can be 
			# IP/MASKBITS or "IP MASK" (2 fields), and different packages
			# specify localhost in different ways
			my $addr = $fields[3];
			if ( $fields[0] =~ /^host/ && $addr =~ /127.0.0.1\/\d+/ )
			{
				foreach my $n qw(0 255)
				{
					my @f = @fields; # modify copy instead of original
					splice @f, 3, 1, qq(127.0.0.1 255.255.255.$n);
					$key = join "-", @f;
					$seen{$key} = 1;
				}
			}
				
			foreach my $a (@REQUIRED_HBA)
			{
				dprint "Existing: @{$a}";
			}
		}
	}
	else ### ! if ( -f "$hba_file" )
	{
		print "Can't read file: $hba_file.  Creating new one.\n";	
	}

	my ($k,$v);
	while (($k,$v) = each(%seen))
	{
		dprint "seen: $k = $v";
	}

	foreach my $item (@REQUIRED_HBA)
	{
		my $key = join "-", @{$item};
		unless ($seen{$key})
		{
			# it's missing, so add to @toadd
			push @toadd, $item;
		}
	}

	if (scalar @toadd > 0)
	{
		dprint "Need to add:";
		foreach my $a (@toadd)
		{
			my $newline = join("\t", @{$a});
			dprint "$newline";
			splice(@lines, $last_blank_line, 0, $newline);
		}
	}
	else
	{
		dprint "$hba_file has the required settings.";
	}

	copy($hba_file, "${hba_file}.pre-wikid");
	open(wFILE, ">", $hba_file) or die "Unable to open '$hba_file' for updating: $!\n";
	foreach my $line (@lines)
	{
		chomp $line;
		print wFILE "$line\n";
	}
	close(wFILE);
}

sub dprint($)
{
	return unless ($DEBUG > 0);
	print STDERR "@_\n";
}

sub restart_postgres
{
	`$WIKID_HOME/sbin/service.sh postgresql stop > /dev/null 2>&1`;
	`$WIKID_HOME/sbin/service.sh postgresql start > /dev/null 2>&1`;
}
	
exit 0;

