#!/usr/bin/perl
## Master control script for the WiKID Authentication Server
##
## Copyright 2001-2014 WiKID Systems, Inc.
## Released under the GNU General Public License, Version 2.
##
## Basic setup function is to set the networking attributes to allow
## further configuration via the http interface.

## Configuration file that is read/written by this script to persist values 
## across invocations:

use strict;
#use warnings;

my $WIKID_HOME="/opt/WiKID";
my $CONF=$WIKID_HOME."/conf/setup.conf";
my $template_dir = $WIKID_HOME."/conf/templates";
my $JAVA_HOME="/opt/java";
my $JAVACMD=$JAVA_HOME."/bin/java";

my $enable = "yes";
my $complete= "n";

my %serverParams;
my %replicant;
my $postgres_port=5432;

my @template = (
   "/etc/sysconfig/network",
   "/etc/resolv.conf",
);
   
my %labels = (
 	"wauth","wAuth protocol daemon",

 	"tomcat","Tomcat webapp server",
	"ldap", "LDAP protocol daemon",
 	"postgres","PostgreSQL database",
	"timecop", "WiKID timecop daemon",
	"logger", "Central logger",
 	);

my %listeners = (
 	"wauth","8388",

 	"tomcat","80",
	"ldap", "389",
 	"postgres","5432",
	"timecop", "8389",
	"logger"."8300",
 	);
 	
my %processes = (
 	"wauth", "com.wikidsystems.server.wAuth",

 	"tomcat", "org.apache.catalina.startup.Bootstrap",
	"ldap", "org.opends.server.core.DirectoryServer",
 	"postgres", "(postmaster|bin/postgres)",
 	"timecop", "com.wikidsystems.server.timecop",
	"logger", "com.wikidsystems.log.CentralLogger",
	);

my %cache = ();
my $display_checks = 0;
my $use_cache = 0;
my $force_yes = 0;
my $check_level = $ENV{'OCF_CHECK_LEVEL'} || 0;

my @arglist = ();
foreach my $arg (@ARGV)
{
	if ($arg eq '-v')
	{
		$display_checks = 1;
	}
	elsif ($arg eq '-q')
	{
		$display_checks = 0;
	}
	elsif ($arg =~ /-c(\d*)/ && !$check_level)
	{
		$check_level = $1;
	}
	elsif ($arg eq '-y')
	{
		$force_yes = 1;
	}
	else
	{
		push @arglist, $arg;
	}
}

my @interfaces = DetermineInterfaces();

#system("clear");
LoadConfig();

my $CMD = $0;
$CMD = `basename $CMD`;
chomp $CMD;
if ($CMD eq 'wikidctl' || $CMD eq 'wikid.pl')
{
	$CMD = shift @arglist;
}

if ($CMD=~/(setup)/i)
{
	# Setup initial networking and replication settings
	print "Starting $1 ...\n";
	setup();
}elsif  ($CMD=~/(start)/i){
	startAll();
}elsif  ($CMD=~/(stop)/i){
	stopAll();
}elsif  ($CMD=~/(demote)/i){
	demote();
}elsif  ($CMD=~/(promote)/i){
	promote();
}elsif  ($CMD=~/(role)/i){
	my $role = $replicant{'role'};
	$role = 'none' unless ($role);
	print "${role}\n";
	exit(0);
}elsif  ($CMD=~/(monitor|status)/i){
	$use_cache = 0;
	status();
}elsif  ($CMD=~/(recover)/i){
	recover();
}elsif  ($CMD=~/(synch?)/i){
	sync();
}elsif  ($CMD=~/(upgrade)/i){
   upgrade(@arglist);
}else{
	print "Unknown Invocation\n";
	exit(3);
}

sub check {
	return check_run(@_) && check_listen(@_);
}

sub check_run {
	# $_[0] = process signature to match via "ps auxww"
	# $_[1] = skip cache flag (0/1)
	# return: 1 if process(es) found, 0 if not
	# return 1 as silent-skip if the service is not enabled
	my $name = $_[0];
	my $skip_cache = $_[1] + 0;
	#print "check_run: name = $name, skip_cache = $skip_cache\n";
	return 1 unless (is_enabled($name));
	my $key = "run_$name";
	return $cache{"$key"} if (!$skip_cache && $use_cache && exists($cache{"$key"})); 
	my $procs=`ps auxww | grep -v grep |egrep '$processes{$name}' |wc -l`;
	$procs=~/^\s*(\d+)/;
	my $result = -1;
	if($procs && $procs>0){
		print "Processes for $labels{$name} OK.\n" if ($display_checks);
		$result = 1;
	}else{
		print "No processes for $labels{$name} found.\n" if ($display_checks);
		$result = 0;
	}
	$cache{"$key"} = $result;
	#print "check_run: result = $result\n";
	return $result;
}

sub check_listen{
	# $_[0] = port name from %listeners to match via "netstat -an"
	# $_[1] = skip cache flag (0/1)
	# return: 1 if process(es) found, 0 if not
	# return 1 as silent-skip if the service is not enabled
	my $name = $_[0];
	my $skip_cache = $_[1] + 0;
	#print "check_listen: name = $name, skip_cache = $skip_cache\n";
	return 1 unless (is_enabled($name));
	my $key = "port_$name";
	return $cache{"$key"} if (!$skip_cache && $use_cache && exists($cache{"$key"})); 
	my $port = $listeners{"$name"};
	if ($name eq 'postgres')
	{
		$port = $postgres_port;
	}
	my $result = -1;
	my $procs=`netstat -an | egrep '(LISTEN|udp)' | grep \":${port}\" | wc -l`;
	$procs=~/^\s*(\d+)/;
	if($procs && $procs>0){
		print "Listening port for $labels{$name} OK.\n" if ($display_checks);
		$result = 1;
	}else{
		print "No listening port for $labels{$name} found.\n" if ($display_checks);
		$result = 0;
	}
	$cache{"$key"} = $result;
	#print "check_listen: result = $result\n";
	return $result;
}

sub recover {
	print "To recover a failed master server from a slave server (Currently \nserving as master):\n
	1) Stop the WiKID services on both servers. (Run 'wikidctl stop' on \n\tboth servers)\n
	2) Run 'wikidctl sync' on the current master and select Both files \n\tand DB.\n
	3) Run 'wikidctl demote' on the current master and make it the new \n\tslave.\n
	4) Run 'wikidctl promote' on this server to make it the master.\n
	5) Run 'wikidctl start' on the new slave server. (former master server)\n
	6) Run 'wikidctl start' on this server to start it as master.\n";
}

sub promote {
	# Change the role of a slave server to a master server
	if ($replicant{"role"} eq "master") {
		print "This system is already a master replication server.\n";
		exit(0);
	}
	if ($force_yes)
	{
	 	stopAll();
	 	$replicant{"role"} = "none";
		$postgres_port = 5432;
	 	#$replicant{"slave_ip"}=$replicant{"master_ip"};
	 	#setupKeys();
	 	SaveConfig();
		exit(0);
	}
	print "All services must be stopped to promote this server.\n\n";
	print "This action will change this server's replication role to master.\n\nIs this what you want? (y/N) ";
	my $resp=getResponse("N");
	if ($resp=~/^y/i) {
	 	stopAll();
	 	$replicant{"role"} = "master";
	 	print "What is the IP address of the slave replication server? ($replicant{\"master_ip\"}):" ; 
	 	$replicant{"slave_ip"}=getResponse($replicant{"master_ip"});
	 	setupKeys();
	 	SaveConfig();
	} else {
		print "Doing nothing\n";
	}
	print "Replication role changed and configuration saved.  You can now start this server with replication role: $replicant{\"role\"} \n";  
	exit(0);
}

sub demote {
	# Change the role of a master server to a slave server
	if ($replicant{"role"} ne "master") {
		print "This action can only be performed on a master replication server.\n\tThis server's current replication role is: $replicant{\"role\"}\n";
		exit(0);
	}
	if ($force_yes)
	{
		stopAll();
		$replicant{"role"} = "slave";
		$replicant{"master_ip"}=$replicant{"slave_ip"};
		SaveConfig();
		exit(0);
	}
	print "All services must be stopped to demote this server.\n\n";
	print "This action will change this server's replication role.\n\nIs this what you want? (y/N) ";
	my $resp=getResponse("N");
	if ($resp=~/^y/i) {
		print "What replication role do you want for this server? (slave, none, Quit) :"; 
		$resp=getResponse("Q");
		if ($resp=~/^Q/i){
			exit(0);
		} elsif ($resp=~/^S/i){
	 		stopAll();
	 		$replicant{"role"} = "slave";
	 		print "What is the IP address of the master replication server? ($replicant{\"slave_ip\"}):" ; 
	 		$replicant{"master_ip"}=getResponse($replicant{"slave_ip"});
	 		SaveConfig();
		} elsif ($resp=~/^N/i){			
	 		stopAll();
	 		$replicant{"role"} = "none";
			$postgres_port = 5432;
	 		SaveConfig();
		}	
		print "Replication role changed and configuration saved.  You can now start this server with replication role: $replicant{\"role\"} \n";  
	} else {
		print "Doing nothing\n";
	}
	exit(0);
}

sub sync {
# Synchronize the slave with the master data

# prompt the user to sync Files, Data, or Both
	my $confirmation = '';
	if ($replicant{"role"} ne "master") {
		print "This action can only be performed on a master replication server.\n\tThis servers current replication role is: $replicant{\"role\"}\n";
		exit(0);
	}
	print "This will synchronize the files and data between the master replication server \nand the slave server.  This is a DESTRUCTIVE OPERATION on the slave server.\n";
	print "What would you like to sync:\n\t(F)iles\n\t(D)atabase\n\t(B)oth\n\t(N)one - this will exit this program.\n\n\tSelection(f,d,b,N): ";
	my $sync_selection=getResponse("N");
	if ($sync_selection =~/^n/i) {
		exit(0);
	} elsif ($sync_selection =~/^f/i) {
		print "This will synchronize the server keys and certificates on the slave server.\n\n";
		print "Are you sure you want to destroy the files on the slave server and replace them \nwith copies from the master server? (N/y): \n";
		$confirmation=getResponse("N");
		if ($confirmation=~/^y/i){
			sync_files();
		} else {
			print "Doing nothing\n";
		}
		exit(0);
	} elsif ($sync_selection =~/^d/i) {
		print "This will synchronize the database data on the slave server.\n\n";
		print "You MUST stop BOTH master and slave servers!  Failure to stop both servers can result in unrecoverable data loss.\n\n";
		print "Are you sure you want to destroy all database data on the slave server and \nreplace it with an exact copy from the master server? (N/y): \n";
		$confirmation=getResponse("N");
		if ($confirmation=~/^y/i){
			sync_db();
		} else {
			print "Doing nothing\n";
		}
		exit(0);
	} elsif ($sync_selection =~/^b/i) {
		print "This will synchronize the database data and server keys and certificates on \nthe slave server.\n\n";
		print "You MUST stop BOTH master and slave servers!  Failure to stop both servers can result in unrecoverable data loss.\n\n";
		print "Are you sure you want to destroy all database data, keys and certificates on \nthe slave server and replace them with an exact copy from the master server? (N/y): \n";
		$confirmation=getResponse("N");
		if ($confirmation=~/^y/i){
			sync_db();
			sync_files();
		} else {
			print "Doing nothing\n";
		}
		exit(0);
	}
	exit(0);
}

sub upgrade {
   my ($release, $build) = @_;
   unless ($release and $build) {
      print STDERR "Must specify release and build as parameters.\n";
      return;
   }
   if (check("postgres") ||
      check("wauth") ||
      check("timecop") ||
      check("tomcat") || 
      check("ldap") 

      )
      {
      print "You must stop all WiKID services before running setup!\nTry running 'wikidctl stop'.\n";
      exit(1);
   }
   exit(system('$WIKID_HOME/sbin/apply_update.pl', @_));
}

sub sync_files{
	print "Synchronizing master files to slave...\n";
	# Give rsync the --delete flag to make the directories perfectly in sync
	Master_cp("$WIKID_HOME/private/","$WIKID_HOME/private/", "--delete");
 	my $secfile = '/etc/WiKID/security';
 	if (-f $secfile ) {
 		Master_cp("$secfile","$secfile");
 	}
	Master_cp("$WIKID_HOME/private/","$WIKID_HOME/private/");
}

sub sync_db{
	print "Synchronizing DB\n";
	print "Ensuring databases are stopped...\n";
	Master_cmd("$WIKID_HOME/sbin/service.sh postgresql stop");
	system("$WIKID_HOME/sbin/service.sh postgresql stop");	
	print "Erasing slave data...\n";
	Master_cmd("rm -rf /var/lib/pgsql/data");
	print "Pushing master data to slave...\n";
#	Master_cp("/var/lib/pgsql/data", "/var/lib/pgsql/", "-r");
	Master_cp("/var/lib/pgsql/data", "/var/lib/pgsql/");
	print "Finalizing sync...\n";
#	Master_cmd("chown -R postgres:postgres /var/lib/pgsql/data");
	Master_cmd("[ -x /sbin/restorecon ] && /sbin/restorecon /var/lib/pgsql/data");
}

sub Master_cmd {
	eval {
		local $SIG{ALRM} = sub { die "Timeout connecting to $replicant{slave_ip}\n" };
		alarm 30; # this should be reasonably long enough to get a response back without delaying the master

		system("ssh -i $WIKID_HOME/private/replication.ssh -q -o 'StrictHostKeyChecking no' root\@"."$replicant{\"slave_ip\"}"." ' $_[0] '");

		alarm 0;
	};
	my $err = $@;
	#if ( $err and $err !~ /Timeout/i ) { warn "$err"; }  # warn only when the error isn't the timeout
	warn "While talking to slave server: $err\n" if ($err);  # print a message for any output
}

sub Master_cp {
	eval {
		local $SIG{ALRM} = sub { die "Timeout connecting to $replicant{slave_ip}\n" };
		alarm 30; # this should be reasonably long enough to get a response back without delaying the master

#		system("scp $_[2] -i $WIKID_HOME/private/replication.ssh -q $_[0] root\@"."$replicant{\"slave_ip\"}:$_[1]");
		system("rsync $_[2] -a -e'ssh -i $WIKID_HOME/private/replication.ssh' -q $_[0] root\@$replicant{\"slave_ip\"}:$_[1]");

		alarm 0;
	};
	my $err = $@;
	#if ( $err and $err !~ /Timeout/i ) { warn "$err"; }  # warn only when the error isn't the timeout
	warn "While talking to slave server: $err\n" if ($err);  # print a message for any output
}


sub startAll {
	# Start all the services
	if ($replicant{"role"} eq "master"){
		# make sure slave DB listener is started ... this will start only if not already started.
		sync_files();
		Master_cmd("$WIKID_HOME/bin/wikidctl start");
	}
	postgres_ctl("start");
	if ($replicant{"role"} ne "slave"){
		logger_ctl("start");
		timecop_ctl("start");
		wauth_ctl("start");
		tomcat_ctl("start");

		ldap_ctl("start");
		firewall_open();
	}
}

sub stopAll {
	# Stop all the services
	if ($replicant{"role"} ne "slave"){
		firewall_close();
		tomcat_ctl("stop");
		timecop_ctl("stop");
		wauth_ctl("stop");

		ldap_ctl("stop");
		logger_ctl("stop");		
	}
	postgres_ctl("stop");
	if ($replicant{"role"} eq "master"){
		# make sure slave DB listener is started ... this will start only if not already started.
		Master_cmd("$WIKID_HOME/bin/wikidctl stop");
	}
}

sub stopMost {
	# Stop all the services
	if ($replicant{"role"} ne "slave"){
		tomcat_ctl("stop");
		timecop_ctl("stop");
		wauth_ctl("stop");

		ldap_ctl("stop");
		logger_ctl("stop");		
	}
	postgres_ctl("stop");
	if ($replicant{"role"} eq "master"){
		# make sure slave DB listener is started ... this will start only if not already started.
		Master_cmd("$WIKID_HOME/bin/wikidctl stop");
	}
}


sub daemon_ctl {
	# $_[0] = daemon
	# $_[1] = start/stop
	service_ctl("$WIKID_HOME/bin/daemonctl $_[1] $_[0]");	
}

sub service_ctl {
	# $_[0] is the command to execute
  # OLD: $_[1] is the output redirection (defaults to hiding all output)
	# $_[1] is deprecated, since we now want to capture output but 
  # only show on errors 
  #print "service_ctl: $_[0] $_[1]\n";
	my $hideout;
	if ( $_[1]  eq "") {
		$hideout = ">/dev/null 2>&1";
	} else {
		$hideout = $_[1];
	}
  #my $rc=open FH, "| $_[0] $hideout";
  my $output = `$_[0]`;
  my $rcRaw = $?;
  my $rc = $rcRaw >> 8; # have to shift to get actual return value
	if ($rc == 0){
		print "Success!\n";
    #close FH;
	} else {
		print "Failed! Returned status $rc, output was:\n";
    print $output;
    #close FH;
	}
}

sub is_db_configured()
{
		my $pgstatus = `$WIKID_HOME/sbin/check_pg.sh`;
		my $dbconfigured = `psql -d wikid -U postgres -h localhost -p $postgres_port -l 2>/dev/null | grep -i wikid`;
		chomp $pgstatus;
		chomp $dbconfigured;
		#print "Status: $pgstatus; ConfiguredCheck: $dbconfigured\n";
		if ($pgstatus eq '0')
		{
			if (!$dbconfigured)
			{
				return 0;
			}
		}

		## TODO: not the best return value, but good enough for now. 
        return 1;
}

sub postgres_ctl {
	if ($_[0] eq "start"){
		if (check("postgres")) {
			print "Database already started.\n";
			return(0);
		}
		if ($replicant{"role"} ne "slave"){
			print "Starting database...";
		} else {
			print "Starting database as replica slave...";
		}
		service_ctl("$WIKID_HOME/sbin/service.sh postgresql start");

		## check if we need to load the WiKID db
        my $db2 = is_db_configured();
        chomp $db2;
        if (!$db2)
        {
                service_ctl("/opt/WiKID/sbin/load_db.sh");
        }

		if ($replicant{"role"} eq "master"){
			print "Starting Replication...";
			my $ssh_result=open(SSH_FILE_HNDL, "ssh -i $WIKID_HOME/private/replication.ssh -N -q -x -o 'StrictHostKeyChecking no' root\@"."$replicant{\"slave_ip\"} -L 5436:localhost:5434 |");
			print "($ssh_result)";
			`echo $ssh_result > $WIKID_HOME/log/ssh_tunnel.pid`;
			service_ctl("$WIKID_HOME/bin/usogres");
			sleep 3;
		}

		## This patch is here to get 3.0 servers up to 3.1.  The plpgsql needed to do
		## the upgrade is incompatible with the migratedb system.  The code in the cumulative
		## patch is resiliant enough to run multiple times without causing errors.
		if(-r "$WIKID_HOME/conf/database/migrations/cumulative.sql") {
			if(! -e "$WIKID_HOME/conf/database/migrations/cumulative.sql.done") {			
				print STDERR "Applying cumulative schema updates...";
				sleep 20;
				open(PSQL_FILE_HNDL, "psql -U postgres -h localhost -d wikid -f $WIKID_HOME/conf/database/migrations/cumulative.sql 2>&1 |");
				open(CLOG,">> $WIKID_HOME/log/cumulative.log");
				while(<PSQL_FILE_HNDL>){
					print CLOG $_;
				}
				close CLOG;
				open(FLAG,">> $WIKID_HOME/conf/database/migrations/cumulative.sql.done");
				print FLAG "Done" ;
				close FLAG; 	
			}
		}
		

		## always run migratedb after starting the DB, to ensure it's up-to-date
		## before the other daemons start.  and let's wait until slave (if used)
		## *should be* started to run the migration scripts.
		daemon_ctl('migratedb');

	} elsif ($_[0] eq "stop") {
		if (!check("postgres")) {
			print "Database already stopped.\n";
			return(0);
		}
		print "Stopping database...";
		if ($replicant{"role"} eq "master"){
			system("/usr/bin/killall -q usogres");
			close(SSH_FILE_HNDL); # does this really work?! 
			if (-f "$WIKID_HOME/log/ssh_tunnel.pid") {
				`kill \`cat $WIKID_HOME/log/ssh_tunnel.pid \`  >>/dev/null 2>&1 `;
			}
		}
		service_ctl("$WIKID_HOME/sbin/service.sh postgresql stop");
	}
}

sub timecop_ctl {
	if ($_[0] eq "start"){
		if (check("timecop")) {
			print "TimeCop process already started.\n";
			return(0);
		}
		print "Starting TimeCop service...";
		if (check_listen("postgres")) {
			daemon_ctl('timecop', 'start');
		} else {
			sleep 3;
			if (check_listen("postgres")) {
				daemon_ctl('timecop', 'start');
			} else {
				print "Database listener not available!  Timecop NOT started!\n";	
				return(1);
			}
		}
	} elsif ($_[0] eq "stop") {
		if (!check("timecop")) {
			print "TimeCop process already stopped.\n";
			return(0);
		}
		if (-f "$WIKID_HOME/log/timecop.pid") {
			my $pid=`cat $WIKID_HOME/log/timecop.pid`;
			if (`ps --no-heading -p $pid ` eq "") {
				print "TimeCop process already Dead.  Check the logs for errors.\n";
			} else { 
				print "Stopping TimeCop service...";		
				daemon_ctl('timecop', 'stop');
				if (-f "$WIKID_HOME/log/timecop.pid") {`rm -f $WIKID_HOME/log/timecop.pid`;}
			}
		} else {
			print "TimeCop process already stopped.\n";
		}
	}	
}

sub logger_ctl {
	if ($_[0] eq "start"){
		if (check("logger")) {
			print "Logger process already started.\n";
			return(0);
		}
		print "Starting Logger service...";
		if (check_listen("postgres")) {
			daemon_ctl('logger', 'start');
		} else {
			sleep 3;
			if (check_listen("postgres")) {
				daemon_ctl('logger', 'start');
			} else {
				print "Database listener not available!  Logger NOT started!\n";	
				return(1);
			}
		}
	} elsif ($_[0] eq "stop") {
		if (!check("logger")) {
			print "Logger process already stopped.\n";
			return(0);
		}
		if (-f "$WIKID_HOME/log/centralLogger.pid") {
			my $pid=`cat $WIKID_HOME/log/centralLogger.pid`;
			if (`ps --no-heading -p $pid ` eq "") {
				print "Logger process already Dead.\n";
			} else { 
				print "Stopping Logger service...";		
				daemon_ctl('logger', 'stop');
				if (-f "$WIKID_HOME/log/centralLogger.pid") {`rm -f $WIKID_HOME/log/centralLogger.pid`;}
			}
		} else {
			print "Logger process already stopped.\n";
		}
	}	
}
sub wauth_ctl {
	return unless (-f "$WIKID_HOME/private/intCAKeys.p12");
	if (!is_enabled('wauth'))
	{
		print "wAuth protocol not yet enabled.\n";
		return 1;
	}
	return unless check('postgres');

	if($_[0] eq "start"){
		if (check("wauth")) {
			print "wAuth protocol daemon already started.\n";
			return(0);
		}
		print "Starting wAuth protocol daemon...\n";
    my $res = system("$WIKID_HOME/bin/daemonctl start wauth");
    $res = $res / 256;
    #print "Result: $res\n";

    if ($res != 0)
    {
      print "\nFailed to start wAuth protocol daemon. Check $WIKID_HOME/log/wauth.log for details.\n";
      stopAll(); # need to return to all-stop state
      exit(1);
    }

		sleep 3;
		if (check("wauth")){
			print "Success!\n";
		} elsif (check_run("wauth") && ! (check_listen("wauth")) ) {
			print "\n\tWaiting for wAuth initialization to complete.";
			while( ! (check_listen("wauth")) ) {
				print "." ;
				sleep 1;
			}
			print "Success!\n";
		} else {
			print "\nStarting wAuth protocol daemon...Failed!\nThe wAuth listener did not start.  Check $WIKID_HOME/log/wauth.log for details.\n";
			if (`grep \"error constructing MAC\"  $WIKID_HOME/log/wauth.log | wc -l`) {
				print "It appears your passphrase was incorrect.  This is the most likely cause of the failure.\n\n";
			}
			#if (-f "$WIKID_HOME/log/wauth.pid") { `rm -f $WIKID_HOME/log/wauth.pid `};
		}
	} elsif ($_[0] eq "stop") {
		if (!check("wauth")) {
			print "wAuth protocol daemon already stopped.\n";
			return(0);
		}
		if (-f "$WIKID_HOME/log/wauth.pid") { 
			my $pid=`cat $WIKID_HOME/log/wauth.pid`;
			if (`ps --no-heading -p $pid` eq "" ) {
				print "wAuth protocol daemon already dead.  Check the logs for errors.\n";
			} else { 
				print "Stopping wAuth protocol daemon...";
				daemon_ctl('wauth', 'stop');
				if (-f "$WIKID_HOME/log/wauth.pid") {`rm -f $WIKID_HOME/log/wauth.pid` };			
			 }			
		} else {
			print "wAuth protocol daemon already stopped.\n";
		}
	}
}

sub tomcat_ctl {
	my $tomcat_cmd = "$WIKID_HOME/tomcat/bin/catalina.sh";
	if ( ! -x $tomcat_cmd ) {
		print "Don't know what command to run to control Tomcat! Please check $WIKID_HOME/tomcat.\n";
		return(1);
	}
	if($_[0] eq "start") {
		if (check("tomcat")) {
			print "Tomcat server already started.\n";
			return(0);
		}
		# if the key store for tomcat's ssl connector doesn't exist, create it.
		if (!(-f "$WIKID_HOME/conf/tomcatKeystore")) {
			system("$WIKID_HOME/sbin/make_tomcat_ssl_cert.sh");
		}	
		print "Starting Tomcat server ..."; 
		daemon_ctl('tomcat', 'start');
	} elsif ($_[0] eq "stop") {
		if (!check("tomcat")) {
			print "Tomcat server already stopped.\n";
			return(0);
		}
		print "Stopping Tomcat server ..."; 
		daemon_ctl('tomcat', 'stop');
		if (-f "/var/lock/subsys/tomcat4") {
			system("rm -f /var/lock/subsys/tomcat4");
		}
	}	
}

sub radius_ctl{
	#Radius coming to wikid-community soon.

}

sub ldap_ctl{

	return unless (-f "$WIKID_HOME/private/localhost.p12");
	if (!is_enabled('ldap'))
	{
		#print "LDAP protocol daemon not available until configured through the admin console.\n";
		print "LDAP protocol not enabled.\n";
		return 1;
	}
	return unless check('postgres');

	if ($_[0] eq "start") {
		if (check("ldap")) {
			print "LDAP protocol daemon already started.\n";
			return(0);
		}
		if (check_listen("wauth")) {
			print "Starting LDAP protocol daemon...";	
			daemon_ctl('ldap', 'start');
		}
		else
		{
			print "LDAP protocol daemon was not started since wAuth not available.\n";
		}
	} elsif ($_[0] eq "stop") {
		if (!check("ldap")) {
			print "LDAP protocol daemon already stopped.\n";
			return(0);
		}
		print "Stopping LDAP protocol daemon ...";	
		daemon_ctl('ldap', 'stop');
	}
}

# prints status of WiKID via format:
# <Status Word>: <Status Message>
# then exits with return code
sub status 
{
# Return Codes from the Linux-HA spec
# OCF_SUCCESS=0
# OCF_ERR_GENERIC=1
# OCF_ERR_ARGS=2
# OCF_ERR_UNIMPLEMENTED=3
# OCF_ERR_PERM=4
# OCF_ERR_INSTALLED=5
# OCF_ERR_CONFIGURED=6
# OCF_NOT_RUNNING=7

# Check for the presence of this environment variable:
# OCF_CHECK_LEVEL
#	0	The most lightweight check possible, which should not
#		have an impact on the QoS.
#		Example: Check for the existence of the process.
#	10	A medium weight check, expected to be called multiple
#		times per minute, which should not have a noticeable
#		impact on the QoS.
#		Example: Send a request for a static page to a
#		webserver.
#	20	A heavy weight check, called infrequently, which may
#		impact system or service performance.
#		Example: An internal consistency check to verify service
#		integrity.

	if ($replicant{"role"} eq "slave") {
		if (check("postgres"))
		{
		  print "Started: WiKID slave services running.\n";
			exit(0);
		}
		else
		{
			print "Stopped: WiKID slave services not running.\n";
			exit(1);
		}

		return;
	}

	my $label = "WiKID" . (($replicant{"role"} eq "master") ? " master" : "") . " services";
	
	if ($check_level) 
	{
		# make sure its a number
		$check_level =~ s/\D//g;
		$check_level += 0;
	}

	# level 0: lightweight = check that the heartbeat servlet is there, which 
	# confirms wauth, tomcat, and postgres
	my $output=`wget -q --output-document=- http://localhost/wikid/HeartBeat`;
	chomp($output);
	if ( !("$output") ) 
	{
		# this means that at least tomcat is stopped, so let's check the others:
		if (!checkAll())
		{
			print "Stopped: $label not running on localhost.\n";
			exit(3);
		}
		print "Stopped: $label not responding on localhost.\n";
		exit(1);
	}
	elsif ($output =~ /failed/io)
	{
		print "Error: $label not functioning properly.\n";
		exit(1);
	}
	elsif ($output =~ /ok/io)
	{
		if ($check_level == 0) 
		{
		  print "Started: $label running.\n";
			exit(0);
		}
	}
	else
	{
		# as of 2007/07/31, this block should never execute
		print "Warning: $output";
	}

	# this is probably the lighter weight check, but the above is more reliable for a quick yes/no
	if ($check_level >= 10)
	{
		if (checkAll())
		{
		  print "Started: $label running.\n";
			exit(0);
		}
		print "Error: Unspecified problem checking $label.\n";
		exit(1);
	}
}

sub checkAll {
	return (check("postgres") &&
					check("wauth") &&
					check("timecop") &&
					check("tomcat")
  			&& check("ldap")

	)
}

sub setup {
	print "Ensuring WiKID services are stopped ...\n";
	stopMost();	
	if (checkAll())
	{
		print "You must stop all WiKID services before running setup!\nUse 'wikidctl stop'.\n";
		exit(1);
	}

	my $config_network = 1;

	$serverParams{"hostname"} = `$WIKID_HOME/sbin/get_hostname.pl`;
        chomp $serverParams{"hostname"};

	my $ifcount = `$WIKID_HOME/sbin/get_interfaces.pl | wc -l`;
	chomp $ifcount;
	print "Found $ifcount interfaces\n";
	if ($ifcount > 0) { 
		print "Looks like networking is already configured.  Do you want to reconfigure? (Y/n) ";
		if (<STDIN> =~ /^n/i){
			$config_network = 0;
		}
	}

	if ($config_network == 1)
	{
		print "\n\n";
		print "Welcome to the initial nework configuration program for the WiKID\n".
		"Authentication Server.  This program will collect some basic information\n".
		"about the local network environment and allow you to define the\n".
		"servers IP address(es).  It will also configure this server to participate\n".
		"in replication if desired.\n".
		"\nNOTICE: You must be root for this to work.\n".
		"\nYou will need the following information to complete this process:\n".
		"\tThe system's fully qualified host name\n".
		"\tAn IP address for each network interface\n".
		"\tThe apropriate network mask for each interface\n".
		"\tThe IP address for the default gateway\n".
		"\tThe IP addresses for the primary and secondary nameservers\n".
		"\nIf you plan to use replication, you'll also need:\n".
		"\tThe role (master or slave) for this server\n".
		"\tThe IP address of the master or slave server\n".
		"\n";
		
		print "Are you ready to begin? (Y/n) ";
		if (<STDIN> =~ /^n/i){
			exit(0);
		}
		
		while (!($complete =~ /^y/i)){
			print "Fully Qualified Domain Name ($serverParams{\"hostname\"}): ";
			$serverParams{"hostname"} = getResponse($serverParams{"hostname"});
			foreach my $if (@interfaces){
				print "Use interface $if->{\"name\"}? (Y/n)";
				if (!(<STDIN> =~ /^n/i)) {
					$if->{"enabled"} = 1;
					print "IP Address for $if->{\"name\"}: ($if->{\"ip\"}): ";
					$if->{"ip"} = getResponse($if->{"ip"});
					print "Network Mask for $if->{\"name\"}: ($if->{\"netmask\"}): ";
					$if->{"netmask"} = getResponse($if->{"netmask"});
				} else {
					$if->{"enabled"} = 0;
				}	
			}
			print "IP Address of Default Gateway ($serverParams{\"default_gw\"}): ";
			$serverParams{"default_gw"} = getResponse($serverParams{"default_gw"});
			print "IP Adress of Primary Nameserver ($serverParams{\"nameserver1\"}): ";
			$serverParams{"nameserver1"} = getResponse($serverParams{"nameserver1"});
			print "IP Adress of Secondary Nameserver ($serverParams{\"nameserver2\"}): ";
			$serverParams{"nameserver2"} = getResponse($serverParams{"nameserver2"});
			ShowList();
			print "Is this correct? (y/N) ";
			$complete = <STDIN>;
		}
	
		ProcessTemplates(@template); 
		writeInterfaces();
		writeHosts();
			
		print "\nStarting Networking...\n";
		foreach my $if (@interfaces){
			if ($if->{"enabled"}){
				print "\nStarting $if->{\"name\"}... \n";
				system("/sbin/ifdown $if->{\"name\"}");
				system("/sbin/ifup $if->{\"name\"}");
			}
		}
	
		print "Setting default route to $serverParams{\"default_gw\"} ...\n";
		system("/sbin/route del default 2> /dev/null");
		system("/sbin/route add default gw $serverParams{\"default_gw\"}");
		system("/sbin/route");
		
		print "Setting hostname to $serverParams{\"hostname\"} ...\n";
		system("/bin/hostname $serverParams{\"hostname\"}");	
	}
	# if the key store for tomcat's ssl connector doesn't exist, create it.
	if (!(-f "$WIKID_HOME/conf/tomcatKeystore")) {
		system("$WIKID_HOME/sbin/make_tomcat_ssl_cert.sh");
	}

	print "Would you like to configure replication? (y/N) ";
	if (<STDIN> =~ /^y/i){
		@template=();
		setupReplication();
		print "Writing replication configuration...\n";
		ProcessTemplates(@template);
	}
	
	SaveConfig();
	
	if ($replicant{"role"} ne "slave") {	
		print "\n\nBasic setup is complete.\nType 'wikidctl start' at the command line to start the WiKID server.\n";
		print "After starting the services, point your web-browser to \n\thttp://$serverParams{\"hostname\"}/WiKIDAdmin/ \nto continue the configuration.\n";
	} else {
		print "\n\nThis server is now configured as a replication slave.\nType 'wikidctl start' at the command line to start the WiKID server.\n";
		print "You can now configure and start the master replication server.\n";
	}
}

sub LoadConfig{
	if (-e $CONF){
		open(CONF,"<$CONF");
		while(<CONF>){
			if(/^#/ || /^\s/){
				next;
			}	elsif (/^interfaces=/) {
				for(my $i=0;$i<$';$i++){
					for(my $config_opts=0;$config_opts<4;$config_opts++){
						my $ln=<CONF>;
						$ln=~/^(\w+)=/;
						$interfaces[$i]->{$1}=$';
						chomp($interfaces[$i]->{$1});
					}
				}
			} elsif (/^role=/) {
				$replicant{"role"}=$';
				chomp $replicant{"role"};
				if ($replicant{"role"} eq "master" || $replicant{"role"} eq "slave"){
						my $ln=<CONF>;
						$ln=~/^(\w+)=/;
						$replicant{$1}=$';
						chomp($replicant{$1});
				}
			} elsif (/^postgres_port=/) {
				$postgres_port=$';
				chomp($postgres_port);
				$listeners{'postgres'} = $postgres_port;
			}else{
				/^(\w+)=/;
				$serverParams{$1}=$';
				chomp($serverParams{$1});
			}
		}
	} else {
		 print "No previous configuration detected.  Starting with factory default values.\n\n";
	}
}

sub SaveConfig{
	open(CONF, ">$CONF") or print "\n**WARNING**: Couldn't cache configuration!\n\n";
	print CONF "## WiKID Systems setup configuration cache\n#!!  DO NOT EDIT THIS FILE  !!\n# If values in this file are incorrect, run setup to correct them\n# You have been advised!\n#################################\n\n";
	print CONF "hostname=$serverParams{\"hostname\"}\n";
	print CONF "default_gw=$serverParams{\"default_gw\"}\n";
	print CONF "nameserver1=$serverParams{\"nameserver1\"}\n";
	print CONF "nameserver2=$serverParams{\"nameserver2\"}\n";
	print CONF "postgres_port=$postgres_port\n";
	print CONF "\n## Detected Interfaces\n";
	print CONF "interfaces=".scalar(@interfaces)."\n";
	foreach my $if (@interfaces){
		print CONF "name=$if->{\"name\"}\n";
		print CONF "enabled=$if->{\"enabled\"}\n";
		print CONF "ip=$if->{\"ip\"}\n";
		print CONF "netmask=$if->{\"netmask\"}\n";
	}
	print CONF "\n## Replication Parameters\n";
	if ($replicant{"role"}=~/^master/i){
		print CONF "role=master\n";	
		print CONF "slave_ip=$replicant{\"slave_ip\"}\n";
	} elsif($replicant{"role"}=~/^slave/i) {
		print CONF "role=slave\n";	
		print CONF "master_ip=$replicant{\"master_ip\"}\n";		
	}else {
		print CONF "role=none\n";	
		print CONF "master_ip_old=$replicant{\"master_ip\"}\n" if $replicant{"master_ip"};
		print CONF "slave_ip_old=$replicant{\"slave_ip\"}\n" if $replicant{"slave_ip"};
	}
	close(CONF);
	savePgConfig();	
}

sub setupKeys {
	if(!( -f "$WIKID_HOME/private/replication.ssh")) {
		print "Generating SSH keys for secure replication...";
		system("/usr/bin/ssh-keygen -b 1024 -f $WIKID_HOME/private/replication.ssh -t dsa -C 'Master Replication Server' -N '' -q");
		print "done!\n";
	}
	print "Adding master SSH public key to the slave server\n";
	my $masterPubKey=`cat $WIKID_HOME/private/replication.ssh.pub`;
	chomp $masterPubKey;
	system("ssh -i $WIKID_HOME/private/replication.ssh -q -o 'StrictHostKeyChecking no' root\@"."$replicant{\"slave_ip\"}"." 'mkdir -p -m 0700 .ssh; echo \"$masterPubKey \" >> .ssh/authorized_keys'");
	#`grep LocalForward /etc/ssh/ssh_config | wc -l`=~/^(\s+)(\d*)/;
	#if ($2 == 0) {
	#	system("echo '        LocalForward 5436 localhost:5434' >> /etc/ssh/ssh_config");
	#}	
}

sub setupReplication {
	## Set the parameters for usogres replication system
	my $rep_setup_done="n";
	while(!($rep_setup_done=~/^y/i)){
		print "Will this system function as master, slave or none? (M/s/n) ";
		$replicant{"role"} = getResponse("master"); 
		if ($replicant{"role"}=~/^m/i){
			$replicant{"role"} = "master";
			print "What is the IP address of the slave server? ($replicant{\"slave_ip\"})";
			$replicant{"slave_ip"} = getResponse($replicant{"slave_ip"}); 
			$replicant{"hba_entry"} = "";
			print "Configuring server as:
				Replication Role: MASTER
				Slave server IP:  $replicant{\"slave_ip\"}\n"; 
		} elsif($replicant{"role"}=~/^s/i){
			$replicant{"role"} = "slave";
			print "What is the IP address of the master server? ($replicant{\"master_ip\"})";
			$replicant{"master_ip"} = getResponse($replicant{"master_ip"});
			$replicant{"hba_entry"} = "host    wikid   $replicant{\"master_ip\"} 255.255.255.255 trust";
			print "Configuring server as:
				Replication Role: SLAVE
				Master server IP: $replicant{\"master_ip\"}\n"; 
		} elsif($replicant{"role"}=~/^n/i) {
            $replicant{"role"} = "none";
            $replicant{"hba_entry"} = "";
            $rep_setup_done = "y";
            print "Configuring server as:
            Replication Role: NONE\n";
		} else {
			print "You must select master or slave. Continue replication configuration? (Y/n)";
			my $cont_setup = getResponse("Y");
			if ($cont_setup=~/^y/i) {
				$rep_setup_done="n";
			} else {
				$replicant{"role"} = "none";
				$replicant{"hba_entry"} = "";
				$rep_setup_done = "y";
				print "Configuring server as:
				Replication Role: NONE\n"; 
			}
		}
		print "Are these values correct? (Y/n)";
		$rep_setup_done = getResponse("y");
	}
	system("echo $replicant{\"role\"} >$WIKID_HOME/conf/replica.conf");
	if ($replicant{"role"} eq "master"){
		$postgres_port = 5434;
		setupKeys();
	} elsif ($replicant{"role"} eq "slave"){
		$postgres_port = 5434;
	} else {
		$postgres_port = 5432;
	}
	savePgConfig();
	push @template, "$WIKID_HOME/conf/usogresrc";

}

sub savePgConfig 
{
	my $etcpg = '/etc/sysconfig/pgsql';
	mkdir "$etcpg" unless ( -d "$etcpg" );
	my $localconf = "$etcpg/postgresql";
	my @lines = ();
	if ( -f "$localconf" )
	{
		open(Pg, "< $localconf");
		@lines = (<Pg>);
		close(Pg);
		@lines = grep(!/^PGPORT=/, @lines);
	}
	unshift @lines, "PGPORT=$postgres_port\n";
	
	open(Pg, ">$localconf");
	foreach (@lines)
	{
		chomp;
		print Pg "$_\n";
	}
	close(Pg);
}


sub getResponse {
	## $_[0] contains the default value
	## return user input if provided else return the default
	my $resp=<STDIN>;
	if($resp=~/\w/) {
		(chomp ($resp));
		return $resp;
	} else {
		return $_[0];
	}
}

sub ProcessTemplates {

	# this is an interim kludge between the old setup and the new one.  I'm sure 
	# there is a better way to do this but I'm too lazy to find it right now.
	
	my $hostname = $serverParams{"hostname"};
	my $nameserver1 = $serverParams{"nameserver1"}; 
	my $nameserver2 = $serverParams{"nameserver2"};
	my $default_gw = $serverParams{"default_gw"};
	my $slave_ip = $replicant{"slave_ip"};
	my $pg_hba_entry = $replicant{"hba_entry"};
	$nameserver2 ||= $nameserver1;
	$|=1;
	foreach(@_){
		unless (-f $_) { print "Skipping $_ since it does not exist on this server\n"; next; }
		print "Processing $_\n";
		my $filename = $_;
		$filename =~ s/^.*\///;
		open(TEMPLATE, "+< $template_dir/$filename") || die "Can't open $template_dir/$filename";
		open(REAL, "> $_") || die "Can't write to $_";
		while(<TEMPLATE>) {
			$_ =~ s/(\$\w+)/$1/eeg;
			print REAL $_;
		}
		close(TEMPLATE);
		close(REAL);
	}
	print "";
	$|=0;
}

sub ShowList {
	print "\n\tThe system's fully qualified host name: $serverParams{\"hostname\"}\n";
	foreach my $if (@interfaces){
		if ($if->{"enabled"}){
			print "\tThe IP address for interface $if->{\"name\"}: $if->{\"ip\"}\n";
			print "\tThe network mask for interface $if->{\"name\"}: $if->{\"netmask\"}\n";
		}
	}
	print "\tThe IP address for the default gateway: $serverParams{\"default_gw\"}
	The IP addresses for the primary nameserver: $serverParams{\"nameserver1\"}
        The IP addresses for the secondary nameserver: $serverParams{\"nameserver2\"}
	\n";
}

sub DetermineInterfaces {
	my @ints;

	my $dns_i = 1;
        my @ifs = `$WIKID_HOME/sbin/get_interfaces.pl`;
        foreach my $inf (@ifs) {
                my $h = {};
                chomp($inf);
                my @a = split(/\|/, $inf);
                foreach my $pair (@a) {
                        my ($k,$v) = split(/=/, $pair);
                        $h->{$k} = $v;
                        if ($k eq 'nameservers') {
                                foreach my $dns (split(',', $v)) {
                                        $serverParams{"nameserver".$dns_i++} = $dns;
                                }
			} elsif ($k eq 'gateway') {
                                $serverParams{"default_gw"} = $v;
                        }
                }
                push @ints, $h;
        }

	if (-f '/etc/resolv.conf') {
		open (F, '/etc/resolv.conf');
		my $i = 1;
		while (<F>) {
			chomp;
			if (/nameserver\s+(\S+)/) {
				$serverParams{"nameserver".$i++} = $1;
			}
		}
	}
	#print "Found ".scalar(@ints)." interfaces\n";
	return @ints;
}

sub writeInterfaces {

	if (-d '/etc/sysconfig/network-scripts') {

		foreach my $if (@interfaces){

			open INT_FILE, ">/etc/sysconfig/network-scripts/ifcfg-$if->{\"name\"}";
			print INT_FILE "DEVICE=$if->{\"name\"}\n";
			if ($if->{"enabled"}){
				print INT_FILE "ONBOOT=yes\n";
			} else {
				print INT_FILE "ONBOOT=no\n";
			}
			print INT_FILE "BOOTPROTO=none\nIPADDR=$if->{\"ip\"}\nNETMASK=$if->{\"netmask\"}\nNAME=$if->{\"name\"}\n";
			close(INT_FILE);
		}
	} elsif (-f '/etc/network/interfaces') {

		open F, "< /etc/network/interfaces";
		my @lines = <F>;
		close F;

		my @newlines = ();
		foreach my $if (@interfaces){
			my $ifname = $if->{"name"};
			my @stanza = ();
			my $wrote_stanza = 0;
			foreach my $l (@lines) {
				if ($l =~ /^auto $ifname/ .. $l =~ /^\s*$/) { # cool trick, eh? :)
					unless (scalar(@stanza)) {
						push @stanza, "auto $ifname\n";
						push @stanza, "iface $ifname inet static\n";
						push @stanza, "  address $if->{\"ip\"}\n";
						push @stanza, "  netmask $if->{\"netmask\"}\n";
						push @stanza, "\n";
					}
				} else {
					if (scalar(@stanza)) {
						push @newlines, @stanza;
						@stanza = ();
						$wrote_stanza = 1;
					}
					push @newlines, $l;
				}
			}
			if (!$wrote_stanza) {
				push @newlines, @stanza;
			}
		}

		open F, "> /etc/network/interfaces";
		foreach my $l (@newlines) { print F $l; }
		close F;
	}
}

sub writeHosts {
	# first read in existing entries
	open HOSTS, "</etc/hosts";
	my @hosts = <HOSTS>;
	close HOSTS;	

	# now write out any entries we need to be there
	open HOSTS, ">>/etc/hosts";
	if ( ! grep /localhost/, @hosts ) {	
		print HOSTS "127.0.0.1\tlocalhost.localdomain localhost\n";
	}
	foreach my $if (@interfaces){
		if ($if->{"enabled"}){
			my $ip = $if->{"ip"};
			if ( ! grep /$ip/, @hosts ) {	
				print HOSTS "$ip\t$serverParams{\"hostname\"}\n";
			}
		}
	}
	close(HOSTS);
}

sub firewall_open {
	return unless check('postgres');
	my @ncs = get_nc_ips();

  #system("/sbin/iptables -t nat -A PREROUTING --protocol tcp --dport 389 --jump REDIRECT --to-port 10389");
  #system("/sbin/iptables -t nat -A OUTPUT -p tcp --dport 389 -j DNAT --to :10389");

	foreach (@ncs) {
		# open the WAS protocol ports for each NC
		system("/sbin/iptables -I INPUT -p tcp -m state --state NEW -j ACCEPT -m tcp --dport 389 -s $_");

		system("/sbin/iptables -I INPUT -p tcp -m state --state NEW -j ACCEPT -m tcp --dport 8388 -s $_");
		# open tacacs+ port
		system("/sbin/iptables -I INPUT -p tcp -m state --state NEW -j ACCEPT -m tcp --dport 49 -s $_");
	}
}

sub firewall_close {
	return unless check('postgres');
	my @ncs = get_nc_ips("all");
	foreach (@ncs) {
		# delete the WAS protocol ports for each NC (even if they've been deleted from the DB)
		system("/sbin/iptables -D INPUT -p tcp -m state --state NEW -j ACCEPT -m tcp --dport 389 -s $_ 2>/dev/null");

		system("/sbin/iptables -D INPUT -p tcp -m state --state NEW -j ACCEPT -m tcp --dport 8388 -s $_ 2>/dev/null");
		# close tacacs+ port
		system("/sbin/iptables -D INPUT -p tcp -m state --state NEW -j ACCEPT -m tcp --dport 49 -s $_ 2>/dev/null");
	}
}

sub get_nc_ips {
	# Usage: get_nc_ips([all]);  
	# If all is passed in, disabled/deleted nc ips will also be returned.
	
	my @ips = ();
	my $pgstatus = `$WIKID_HOME/sbin/check_pg.sh`;
	my $dbconfigured = `psql -d wikid -U postgres -h localhost -p $postgres_port -l 2>/dev/null | grep -i wikid`;
	chomp $pgstatus;
	chomp $dbconfigured;
	#print "Status: $pgstatus; ConfiguredCheck: $dbconfigured\n";
	if ($pgstatus eq '0')
	{
		if ($dbconfigured)
		{
			my $table = 'network_client';
			if ($_[0] eq "all"){
				$table = 'full_network_client';
			}
			@ips =`psql -U postgres -h localhost -p $postgres_port -d wikid -A -t -c \"select distinct ip from $table\" 2>& 1`;
			#print "IPS = @ips\n";
		}
		else
		{
			warn "The database is not yet configured for WiKID. Please run $WIKID_HOME/sbin/load_db.sh.\n";
			@ips = ();
		}
	}
	else
	{
		warn "The database is down; the firewall cannot be configured for network clients.\n";
		@ips = ();
	}	

	if (scalar @ips > 0)
	{
		if ($ips[0] =~ /(FATAL|ERROR|psql:)/)
		{
			@ips = ();
			die "Error reading network client IPs from database!\n";
		}
	}

	chomp @ips;

	#die "Couldn't read from database!" unless (scalar @ips);
	# TODO: need to check return value here too,
	# because the backticks could be "successful" while returning something like: 
	#    "sh: psql: command not found"
	#    "psql: FATAL:  database "wikid" does not exist"
	# - csidi, Aug2005

	return @ips;
}
 
sub is_enabled
{
	my $check = $_[0];
	#print "Checking if $check is enabled...\n";
	return 1 if ($check =~ /(tomcat|postgres|timecop|logger)/oi);
	my $key = "enabled_$check";	
	return $cache{"$key"} if (exists($cache{"$key"})); 
  my $q = "select h.enabled from host_type ht LEFT OUTER JOIN host h USING (type) where lower(ht.name)='$_[0]';";
  my $status =`psql -U postgres -h localhost -p $postgres_port -d wikid -A -t -c \"$q\" 2>& 1`;
	chomp $status;
  #print "Status of $check: $status\n";
	$status = 0 unless (defined($status));
	$cache{$key} = $status;
	return $status;	
}


